#include <math.h>
#include <stdlib.h>
#include "nrutil.c"
#define float double

float qgaus(float (*func)(float), float a, float b)
/*
Returns the integral of the function func between a and b, by ten-point Gauss-Legendre integration:
the function is evaluated exactly ten times at interior points in the range of integration.
*/
{
	int j;
	float xr,xm,dx,s;
	static float x[] = {0.0,0.1488743389,0.4333953941,		// The abscissas and weights. First value of each array not used.
						0.6794095682,0.8650633666,0.9739065285};
	static float w[] = {0.0,0.2955242247,0.2692667193,
						0.2190863625,0.1494513491,0.0666713443};
	xm=0.5*(b+a);
	xr=0.5*(b-a);
	s=0;				// Will be twice the average value of the function, since the ten weights (five numbers above each used twice) sum to 2.
	for (j=1;j<=5;j++) {
		dx=xr*x[j];	
		s += w[j]*((*func)(xm+dx)+(*func)(xm-dx));
	}
	return s *= xr;					// Scale the answer to the range of integration.
}


void main (int argc, char *argv[])
{

	int i;
	float a = 0.0, b = 3.1416, s = 0.0;

	for (i=1; i<argc; i++ ){
		
		switch(i){
	    	case 1: 
			a = atof(argv[1]); 
			break;
		case 2:
			b = atof(argv[2]); 
			break;
		} 
	}

	if    ( a < b ) s = qgaus( cos, a, b);
	else if ( b < a ) s = - qgaus( cos, b, a);
	
	printf("\n\n\n\t %g\n", b);
	printf("\t /\n");
	printf("\t | cos (x) dx = %.16f \t (Gauss - Legendre)\n", s);
	printf("\t /\n");
	printf("\t %g\n", a);

}
