#include <math.h>
#include <stdlib.h>
#include "nrutil.c"
//#define float double

#define MCPER 10000
#define MCEPS 1.0e-6

#define IM1 2147483563
#define IM2 2147483399
#define AM (1.0/IM1)
#define IMM1 (IM1-1)
#define IA1 40014
#define IA2 40692
#define IQ1 53668
#define IQ2 52774
#define IR1 12211
#define IR2 3791
#define NTAB 32
#define NDIV (1+IMM1/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)


float ran2(long *idum)
/*
Long period (> 2  1018) random number generator of LEcuyer with Bays-Durham shu.e
and added safeguards. Returns a uniform random deviate between 0.0 and 1.0 (exclusive of
the endpoint values). Call with idum a negative integer to initialize; thereafter, do not alter
idum between successive deviates in a sequence. RNMX should approximate the largest .oating
value that is less than 1.
*/
{
int j;
long k;
static long idum2=123456789;
static long iy=0;
static long iv[NTAB];
float temp;
if (*idum <= 0) {				// Initialize.
if (-(*idum) < 1) *idum=1;		// Be sure to prevent idum = 0.
else *idum = -(*idum);
idum2=(*idum);
for (j=NTAB+7;j>=0;j--) {		// Load the shu.e table (after 8 warm-ups).
k=(*idum)/IQ1;
*idum=IA1*(*idum-k*IQ1)-k*IR1;
if (*idum < 0) *idum += IM1;
if (j < NTAB) iv[j] = *idum;
}
iy=iv[0];
}
k=(*idum)/IQ1;					// Start here when not initializing.
*idum=IA1*(*idum-k*IQ1)-k*IR1;	// Compute idum=(IA1*idum) % IM1 without
								// over.ows by Schrages method. 
if (*idum < 0) *idum += IM1;
k=idum2/IQ2;
idum2=IA2*(idum2-k*IQ2)-k*IR2;	// Compute idum2=(IA2*idum) % IM2 likewise.
if (idum2 < 0) idum2 += IM2;
j=iy/NDIV;						// Will be in the range 0..NTAB-1.
iy=iv[j]-idum2;					// Here idum is shu.ed, idum and idum2 are
								// combined to generate output. 
iv[j] = *idum;
if (iy < 1) iy += IMM1;
if ((temp=AM*iy) > RNMX) return RNMX; // Because users dont expect endpoint values.
else return temp;
}

float qmc (double (*func)(double), float a, float b, float fmin, float fmax)
/*
Returns the integral of func from a to b via Monte-Carlo method.
*/
{
	float xr, yr, s, so, ri;
	int i, nt, nf;
	long rl;

	nt = 0;
	nf = 0;
	xr = b-a;
	yr = fmax-fmin;
	s  = 0.0;
	so = 0.0;
	rl = -1;

	ri = ran2(&rl);
	rl = 1;
	do 
	{
		for (i=0; i<MCPER; i++) 
		{
			nt++;
			if ( (yr*ran2(&rl)) <= (*func)( (xr*ran2(&rl)) ) - fmin ) 
				nf++;
		}
		so = s;
		s = ((double)nf)/((double)nt);

	} while (fabs(s-so)<MCEPS*fabs(s) && nt>2*MCPER);

	return (s*xr*yr + fmin*xr);
}

void main (int argc, char *argv[])
{

	int i;
	float a = 0.0, b = 3.1416, s = 0.0;

	for (i=1; i<argc; i++ ){
		
		switch(i){
	    	case 1: 
			a = atof(argv[1]); 
			break;
		case 2:
			b = atof(argv[2]); 
			break;
		} 
	}

	if    ( a < b ) s = qmc( cos, a, b, -1, 1);
	else if ( b < a ) s = - qmc( cos, b, a, -1, 1);
	
	printf("\n\n\n\t %g\n", b);
	printf("\t /\n");
	printf("\t | cos (x) dx = %.16f \t (Monte-Carlo)\n", s);
	printf("\t /\n");
	printf("\t %g\n", a);

}
